/* -*- indent-tabs-mode: t; tab-width: 4; c-basic-offset: 4; -*- */
/*
   pref_window.c
   Projet GtkCmpHash

   (C) Copyright 2007 Olivier Delhomme
   e-mail : olivierdelhomme@gmail.com
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.           
*/

#include "gtkcmphash.h"

static void close_pref_window(GtkWidget *widget, gpointer data);
static void toggle_include_dir(GtkWidget *widget, gpointer data);
static void connect_signaux_pref_window(main_struct_t *main_struct);


/**
 *  Affiche ou cache la fentre des prfrences (modale)
 */
static void show_hide_pref_window(main_struct_t *main_struct, gboolean show)
{
  if (show)
    gtk_widget_show(GTK_WIDGET(glade_xml_get_widget(main_struct->xml, "pref_window")));
  else
    gtk_widget_hide(GTK_WIDGET(glade_xml_get_widget(main_struct->xml, "pref_window")));
}


/**
 *  Pour afficher la fentre des prfrences
 */
void show_pref_window(GtkWidget *widget, gpointer data)
{
  main_struct_t *main_struct = (main_struct_t *) data;
 
  show_hide_pref_window(main_struct, TRUE);
}


/**
 *  Lorsqu'on clique sur le bouton "Fermer" de la fentre des prfrences
 */
static void close_pref_window(GtkWidget *widget, gpointer data)
{
  main_struct_t *main_struct = (main_struct_t *) data;
 
  show_hide_pref_window(main_struct, FALSE);
}

/**
 *  Lorsque l'on change l'option d'inclusion du chemin absolu
 */ 
static void toggle_include_dir(GtkWidget *widget, gpointer data)
{
  main_struct_t *main_struct = (main_struct_t *) data;
  GtkToggleButton *cb_include_dir = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_include_dir"));
  
  main_struct->opts->include_dir = gtk_toggle_button_get_active(cb_include_dir);
}

/**
 *  Pour inclure le nom du hashset dans les rsultats des fihciers connus
 */
static void toggle_include_hashset_name(GtkWidget *widget, gpointer data)
{
  main_struct_t *main_struct = (main_struct_t *) data;
  GtkToggleButton *button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_include_hashset_name"));
  
  main_struct->opts->include_hashset_name = gtk_toggle_button_get_active(button);
}

/**
 *  Pour inclure le nom du fichier ayant le mme hash que celui recherch
 */
static void toggle_include_hashset_file_filename(GtkWidget *widget, gpointer data)
{
  main_struct_t *main_struct = (main_struct_t *) data;
  GtkToggleButton *button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_include_hashset_file_filename"));
  
  main_struct->opts->include_hashset_file_filename = gtk_toggle_button_get_active(button);
}

/**
 *  Pour gnrer, ou non, les hashs des fichiers vides (de taille nulle)
 *  Par dfaut on ne gnre pas
 */
static void toggle_genere_hashs_vides(GtkWidget *widget, gpointer data)
{
  main_struct_t *main_struct = (main_struct_t *) data;
  GtkToggleButton *button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_genere_hashs_vides"));
  
  main_struct->opts->genere_hashs_vides = gtk_toggle_button_get_active(button);
}

/**
 *  Chargement, ou non, des hashs issus de fichiers vides (de taille nulle) 
 *  lors du chargement des hashsets
 */
static void toggle_charger_fv_hashsets(GtkWidget *widget, gpointer data)
{
  main_struct_t *main_struct = (main_struct_t *) data;
  GtkToggleButton *button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_charger_fv_hashsets"));
  
  main_struct->opts->charger_fv_hashsets = gtk_toggle_button_get_active(button);
}


/**
 *  Changement des rsultats de comparaison
 */
static void toggle_all_known(GtkWidget *widget, gpointer data)
{
  main_struct_t *main_struct = (main_struct_t *) data;
  GtkToggleButton *cb_all_known = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_all_known"));

  /*
    On doit supprimer les listes de comparaison, vu que la mthode est 
    diffrente. Pas besoin de tests sur la liste elle mme vu que c'est 
    ralis dans la fonction free_file_hash_list
  */
  free_file_hash_list(main_struct->dedans_ou_pas->found);
  main_struct->dedans_ou_pas->found = NULL;
  free_file_hash_list(main_struct->dedans_ou_pas->not_found);
  main_struct->dedans_ou_pas->not_found = NULL;
    
  main_struct->opts->all_known = gtk_toggle_button_get_active(cb_all_known);
}


/**
 *  Connexion des signaux correspondant  la fentre des prfrences
 */
static void connect_signaux_pref_window(main_struct_t *main_struct)
{
  /* Bouton Fermer */
  g_signal_connect(G_OBJECT(glade_xml_get_widget(main_struct->xml, "pw_quit")), "clicked", 
		   G_CALLBACK(close_pref_window), main_struct);

  /* Option d'inclusion des rpertoires */
  g_signal_connect(G_OBJECT(glade_xml_get_widget(main_struct->xml, "cb_include_dir")), "toggled", 
		   G_CALLBACK(toggle_include_dir), main_struct);

  /* Option d'inclusion de tous les hashset_name dans lesquel apparait le fichier (forcment connu) */
  g_signal_connect(G_OBJECT(glade_xml_get_widget(main_struct->xml, "cb_all_known")), "toggled", 
		   G_CALLBACK(toggle_all_known), main_struct);

  /* Option d'inclusion du nom du hashset pour les fichiers connus */
  g_signal_connect(G_OBJECT(glade_xml_get_widget(main_struct->xml, "cb_include_hashset_name")), "toggled", 
		   G_CALLBACK(toggle_include_hashset_name), main_struct);

  /* Option d'inclusion du nom du fichier (celui de l'ensemble des hashsets) pour lequel le hash est identique */
  g_signal_connect(G_OBJECT(glade_xml_get_widget(main_struct->xml, "cb_include_hashset_file_filename")), "toggled", 
		   G_CALLBACK(toggle_include_hashset_file_filename), main_struct);

  /* Option pour hasher les fihciers vides (de taille nulle) */
  g_signal_connect(G_OBJECT(glade_xml_get_widget(main_struct->xml, "cb_genere_hashs_vides")), "toggled", 
		   G_CALLBACK(toggle_genere_hashs_vides), main_struct);

  /* Option de chargement des hashs issus de fichiers vides */
  g_signal_connect(G_OBJECT(glade_xml_get_widget(main_struct->xml, "cb_charger_fv_hashsets")), "toggled", 
		   G_CALLBACK(toggle_charger_fv_hashsets), main_struct);

}


/**
 *  Initialise la fentre de prfrences avec les valeurs par dfaut
 *  Connecte les signaux
 */
void init_pref_window_with_defaults(main_struct_t *main_struct)
{ 
  GtkToggleButton *button = NULL;

  button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_include_dir"));
  gtk_toggle_button_set_active(button, main_struct->opts->include_dir);

  button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_all_known"));
  gtk_toggle_button_set_active(button, main_struct->opts->all_known);

  button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_include_hashset_name"));
  gtk_toggle_button_set_active(button, main_struct->opts->include_hashset_name);

  button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_include_hashset_file_filename"));
  gtk_toggle_button_set_active(button, main_struct->opts->include_hashset_file_filename);

  button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_genere_hashs_vides"));
  gtk_toggle_button_set_active(button, main_struct->opts->genere_hashs_vides);

  button = GTK_TOGGLE_BUTTON(glade_xml_get_widget(main_struct->xml, "cb_charger_fv_hashsets"));
  gtk_toggle_button_set_active(button, main_struct->opts->charger_fv_hashsets);

  connect_signaux_pref_window(main_struct);
}




